/* 
  Copyright (C) 1999 Pawel Gajda (mis@k2.net.pl)

  This file is part of the PLD Install Library

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.
 
  You should have received a copy of the GNU General Public License along
  with this program; if not, write to the Free Software Foundation, Inc.,
  59 Place - Suite 330, Boston, MA 02111-1307, USA.  
*/

/*
  $Id: misc.c,v 1.5 2001/03/02 19:58:18 pawelk Exp $
*/

#include <errno.h>
#include <limits.h>
#include <stdarg.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <sys/utsname.h>
#include <dirent.h>
#include <fnmatch.h>
#include "md5.h"

#ifdef USE_XMALLOCS
# include <trurl/xmalloc.h>
#endif

#include <trurl/nassert.h>
#include <trurl/narray.h>

#include "log.h"
#include "rpmpldi.h"

int is_dir(const char *path)
{
    struct stat st;
    return stat(path, &st) == 0 && S_ISDIR(st.st_mode);
}


int backup_file(const char *pathname) 
{
    int rc = 1;

    if (access(pathname, R_OK) == 0) {
        char backup[PATH_MAX];

        if (is_dir(pathname)) {
            m_assert(0);
            log_message(_("Cannot rename %s - it is directory!"), pathname);
        }
        
	log_message(_("%s already exists, save it as %s-orig"), pathname,
                    pathname);
        
        strcpy(backup, pathname);
        strcat(backup, ".orig");
        if (rename(pathname, backup) != 0) {
            rc = 0;
            log_message(_("rename failed:%s"), strerror(errno));
        }
    } else {
        log_message(_("%s does not exist"), pathname);
    }
    
    return rc;
}

int mkdir_ifnone(const char *path)
{
    int rc;
    if (access(path, X_OK) == 0) {
        if (!is_dir(path)) {
            log_message(_("Cannot make directory %s: _file_ named %s exists!"));
            return 0;
        }
        return 1;
    }

    log_message(_("Creating directory %s"), path);
    rc = mkdir(path, 0755);
    if (rc)
        log_message(_("mkdir failed: %s"), strerror(errno));

    return rc == 0;
}


int mkdirhier(const char *path) 
{
    char *token;
    char *wr_path, *tmp;
    int len;

    m_assert(path);
    len = strlen(path);
    
    wr_path = alloca(len);
    tmp  = alloca(len);
    
    if (wr_path == NULL || tmp == NULL) {
        return 0;
    }
    strcpy(wr_path, path);
    
    if (*wr_path == '/') {
        *tmp = '/';
        *(tmp+1) = '\0';
        
    } else {
        *tmp = '\0';
    }
    token = strtok(wr_path, "/");
        
    while(token != NULL) {
        strcat(tmp, token);
        
        if (!mkdir_ifnone(tmp)) {
            return 0;
	}
	    
        
        strcat(tmp, "/");
        token = strtok(NULL, "/");
    }
    return 1;
}


char *buildpath(const char *dir, ...) 
{
    static char path[PATH_MAX];
    register int len = 0;
    register char *p;
    va_list ap;

    path[0] = '\0';
    
    if (*dir == '/') {
        while(*dir == '/')
            dir++;
        path[0] = '/';
        path[1] = '\0';
    }
    
    if (*dir)
        strcat(path, dir);
    
    len = strlen(path);
    p = &path[len-1];
        
    if (*p == '/') {
        while(*--p == '/')
            ;
    }
    p++;
    *p++ = '/';
    *p = '\0';
    
    
    va_start(ap, dir);

    while((p = va_arg(ap, char*)) != NULL) {

        if (*p == '/') {
            while(*p == '/')
                p++;
        }
        strcat(path, p);
        len = strlen(path);
        
        p = &path[len-1];
        
        if (*p == '/') {
            while(*--p == '/')
                ;
        }
        p++;
        *p++ = '/';
        *p = '\0';
    }
    
    va_end(ap);
    
    /* chop last '/' */
    len = strlen(path);
    p = &path[len-1];
        
    if (*p == '/') 
        *p = '\0';
    
    return path;
}

#if 1
int find(tn_array **flist, const char *path, const char *pattern)
{
    int n;
    struct dirent *de;
    DIR *dir;
    tn_array *fl = *flist;

    if ((dir = opendir(path)) == NULL) {
        log_message("open %s:", path, strerror(errno));
        return -1;
    }

    n = 0;

    while( (de = readdir(dir)) != NULL ) {
        if (de->d_name[0] == '.' || de->d_name[0] == '\0')
            continue;

        if (pattern) 
            if (fnmatch(pattern, de->d_name, 0) != 0)
                continue;
        
        if (fl == NULL) 
            fl = n_array_new(2, free, (t_fn_cmp)strcmp); 
        
        if (n_array_push(fl, strdup(de->d_name)) == NULL) {
            n_array_free(fl);
            n = -1;
            break;
            
        } else {
            n++;
        }
        
    }
    
    *flist = fl;
    closedir(dir);
    return n;
}
#endif

const char *pldi_basename(const char *path)
{
    char *s = strrchr(path, '/');
    return s == NULL ? path : ++s;
}

#if 0
char *machine_type(char *mtype)
{
    struct utsname buf;
    
    if (uname(&buf) != 0) {
        mtype = NULL;
    } else {
        strcpy(mtype, buf.machine);
    }

    return mtype;
}
#endif

char *getMD5File(int fd)
{
	static char ret[33];
	struct MD5Context ctx;
	static char *hex = "0123456789abcdef";
	int i;
	unsigned char res[16];
	char *buf;
	struct stat st;
	off_t orgOff;

	ret[0]='\0';

	if(fstat(fd,&st)){
		log_message("Can't stat file (for md5 sum)");
		return ret;
	}
	orgOff=lseek(fd,0,SEEK_CUR);
	lseek(fd,0,SEEK_SET);
	buf=malloc(st.st_size);
	read(fd,buf,st.st_size);

	MD5Init(&ctx);
	MD5Update(&ctx,buf,st.st_size);
	MD5Final(res,&ctx);

	ret[32]='\0';
	for(i=0;i<16;i++){
		ret[i*2+1]   = hex[res[i] & 0xf];
		ret[i*2] = hex[(res[i] >>4 ) & 0xf];
	}
	free(buf);
	lseek(fd,orgOff,SEEK_SET);
	return ret;
}

void remMultiSl(char *b)
{
	int i,j=0,skip=0;;

	for(i=0,j=0;j<strlen(b);j++){
		if(b[j]=='/'){
			if(skip==0)
				skip=1;
			else
				continue;
		}
		else
			skip=0;
		b[i]=b[j];
		i++;
	}
	b[i]=0;
}
